/*****************************************************************************
 *  Rohde&Schwarz MXO Oscilloscope instrument driver
 *  Callbacks
 *
 *  Original Release: November 2022
 *  By: Tereza Zavodna
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *

 *****************************************************************************/

#include "rsmxo.h"
#include <stdlib.h>
#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
#include <sys/timeb.h>
#include <sys/types.h>
#include <unistd.h>
#define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif

#pragma warning (disable:4100) // unreferenced formal parameter


 /*****************************************************************************
 *- Read Callbacks ----------------------------------------------------------*
 *****************************************************************************/


 /*****************************************************************************
  * Function: rsmxo_ConvertFromBinary_ReadCallback
  * Purpose:  This function overrides standard calback funtion. Converts
  *           binary number
  *****************************************************************************/
ViStatus rsmxo_ConvertFromBinary_ReadCallback(ViSession          instrSession,
                                              ViConstString      repCapName,
                                              RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  int32Value;
    ViInt64  int64Value;
    ViChar*  cmd = NULL;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    // If no command available, exit the function
    if (attr->command == NULL)
        return VI_SUCCESS;

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));
    checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, response));

    switch (attr->dataType)
    {
    case RS_VAL_INT32:
        int32Value = (ViInt32)RsCore_Convert_BinaryString_To_ViInt64(response);
        checkErr(RsCore_SetAttributeP2Value(instrSession, attr, &int32Value));
        break;
    case RS_VAL_INT64:
        int64Value = RsCore_Convert_BinaryString_To_ViInt64(response);
        checkErr(RsCore_SetAttributeP2Value(instrSession, attr, &int64Value));
        break;
    default:
        viCheckErrElab(RS_ERROR_INVALID_TYPE,
            "rsmxo_ConvertFromBinary_ReadCallback only accepts RS_VAL_INT32 and RS_VAL_INT64 data types");
    }

Error:
    if (cmd)
        free(cmd);

    return error;
}

/****************************************************************************************************
 *  rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback
Replaces <Frame> with value of ViAttr selectedFrame
Replaces <Field> with value of ViAttr selectedField
Replaces <Error> with value of ViAttr selectedError
    ******************************************************************************************************/
ViStatus rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value,
                            ViAttr             selectedFrame,
                            ViAttr             selectedField,
                            ViAttr             selectedError)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   aux[RS_MAX_SHORT_MESSAGE_BUF_SIZE];
    ViInt32  field, frame, _error;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (selectedFrame > 0)
    {
        // Replaces <Frame> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedFrame, 0, &frame));
        snprintf(aux, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld", frame);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Frame>", aux, VI_TRUE);
    }

    if (selectedField > 0)
    {
        // Replaces <Field> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedField, 0, &field));
        snprintf(aux, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld", field);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Field>", aux, VI_TRUE);
    }

    if (selectedError > 0)
    {
        // Replaces <Error> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedError, 0, &_error));
        snprintf(aux, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld", _error);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Error>", aux, VI_TRUE);
    }

    if (attr->rangeTableDiscrete)
    {
        ViChar* p2cmdString = NULL;
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, attr->rangeTableDiscrete, *(ViInt32*)value, &p2cmdString));
        RsCore_StrCatDynamicString(instrSession, &cmd, " ", p2cmdString, NULL);
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            RsCore_StrCatDynamicViInt32(instrSession, &cmd, " ", *(ViInt32*)value, NULL);
            break;
        case RS_VAL_INT64:
            RsCore_StrCatDynamicViInt64(instrSession, &cmd, " ", *(ViInt64*)value, NULL);
            break;
        case RS_VAL_REAL64:
            RsCore_StrCatDynamicViReal64(instrSession, &cmd, " ", *(ViReal64*)value, NULL);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, " \'", (ViString)value, "\'");
            break;
        case RS_VAL_BOOLEAN:
            RsCore_StrCatDynamicViBoolean(instrSession, &cmd, " ", *(ViBoolean*)value, NULL);
            break;
        case RS_VAL_EVENT:
            break;
        case RS_VAL_UNKNOWN_TYPE:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}


/****************************************************************************************************
 *  rsmxo_FormatUniversalAmplitudePointRepCap_WriteCallback
Replaces <AmplitudePoint> with value of ViAttr selectedAmplitudePoint
    ******************************************************************************************************/
ViStatus rsmxo_FormatUniversalAmplitudePointRepCap_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value,
                            ViAttr             selectedAmplitudePoint)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   aux[RS_MAX_SHORT_MESSAGE_BUF_SIZE];
    ViInt32  amplitudePoint;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (selectedAmplitudePoint > 0)
    {
        // Replaces <AmplitudePoint> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedAmplitudePoint, 0, &amplitudePoint));
        snprintf(aux, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld", amplitudePoint);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<AmplitudePoint>", aux, VI_TRUE);
    }

    if (attr->rangeTableDiscrete)
    {
        ViChar* p2cmdString = NULL;
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, attr->rangeTableDiscrete, *(ViInt32*)value, &p2cmdString));
        RsCore_StrCatDynamicString(instrSession, &cmd, " ", p2cmdString, NULL);
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            RsCore_StrCatDynamicViInt32(instrSession, &cmd, " ", *(ViInt32*)value, NULL);
            break;
        case RS_VAL_INT64:
            RsCore_StrCatDynamicViInt64(instrSession, &cmd, " ", *(ViInt64*)value, NULL);
            break;
        case RS_VAL_REAL64:
            RsCore_StrCatDynamicViReal64(instrSession, &cmd, " ", *(ViReal64*)value, NULL);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, " \'", (ViString)value, "\'");
            break;
        case RS_VAL_BOOLEAN:
            RsCore_StrCatDynamicViBoolean(instrSession, &cmd, " ", *(ViBoolean*)value, NULL);
            break;
        case RS_VAL_EVENT:
            break;
        case RS_VAL_UNKNOWN_TYPE:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}


/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CFrameRepCapBoolean_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CFrameRepCapBoolean_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CFrameFieldRepCapBoolean_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CFrameFieldRepCapBoolean_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPMIFrameRepCapBoolean_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPMIFrameRepCapBoolean_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPIFrameFieldRepCapBoolean_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPMIFrameFieldRepCapBoolean_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPIFrameFieldRepCapInt32_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPMIFrameFieldRepCapInt32_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolTNOSFrameFieldRepCapInt32_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolTNOSFrameFieldRepCapInt32_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CFrameFieldRepCapInt32_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CFrameFieldRepCapInt32_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolCANFrameFieldRepCapInt32_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANFrameFieldRepCapInt32_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolCANFrameFieldRepCapReal64_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANFrameFieldRepCapReal64_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FIELD, 0);
}


/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CFrameRepCapString_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CFrameRepCapString_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPIErrorRepCapBoolean_WriteCallback
Replaces <Error> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_ERROR
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPIErrorRepCapBoolean_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        0, 0, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_ERROR);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolTNOSErrorRepCapBoolean_WriteCallback
Replaces <Error> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_ERROR
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolTNOSErrorRepCapBoolean_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        0, 0, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_ERROR);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CErrorRepCapBoolean_WriteCallback
Replaces <Error> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_ERROR
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CErrorRepCapBoolean_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        0, 0, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_ERROR);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolUARTFrameRepCapBoolean_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolUARTFrameRepCapBoolean_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_UART_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolHBTOFrameRepCapBoolean_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolHBTOFrameRepCapBoolean_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPIFrameRepCapBoolean_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPIFrameRepCapBoolean_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPIFrameFieldRepCapBoolean_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPIFrameFieldRepCapBoolean_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolUARTFrameFieldRepCapBoolean_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolUARTFrameFieldRepCapBoolean_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_UART_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_UART_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolUARTFrameFieldRepCapInt32_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolUARTFrameFieldRepCapInt32_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_UART_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_UART_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolHBTOFrameFieldRepCapInt32_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolHBTOFrameFieldRepCapInt32_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPIFrameFieldRepCapInt32_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPIFrameFieldRepCapInt32_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolUARTErrorRepCapBoolean_WriteCallback
Replaces <Error> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_ERROR
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolUARTErrorRepCapBoolean_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(instrSession, repCapName, attr, value,
        0, 0, RSMXO_ATTR_PROTOCOL_UART_SELECTED_ERROR);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolCANFrameRepCapInt32_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANFrameRepCapInt32_WriteCallback(
                ViSession io,
                ViConstString repCapName,
                RsCoreAttributePtr attr,
                void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolCANFrameRepCapReal64_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANFrameRepCapReal64_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolCANFrameRepCapString_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANFrameRepCapString_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatFRANAmplitudePointRepCapAction_WriteCallback
Replaces <AmplitudePoint> with value of RSMXO_ATTR_SELECTED_FREQUENCY_RESPONSE_ANALYSIS_AMPLITUDE_PROFILE_POINT
******************************************************************************************************/
ViStatus rsmxo_FormatFRANAmplitudePointRepCapAction_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalAmplitudePointRepCap_WriteCallback(io, repCapName, attr, value,
        RSMXO_ATTR_SELECTED_FREQUENCY_RESPONSE_ANALYSIS_AMPLITUDE_PROFILE_POINT);
}

/****************************************************************************************************
 *  rsmxo_FormatFRANAmplitudePointRepCapReal64_WriteCallback
Replaces <AmplitudePoint> with value of RSMXO_ATTR_SELECTED_FREQUENCY_RESPONSE_ANALYSIS_AMPLITUDE_PROFILE_POINT
******************************************************************************************************/
ViStatus rsmxo_FormatFRANAmplitudePointRepCapReal64_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalAmplitudePointRepCap_WriteCallback(io, repCapName, attr, value,
        RSMXO_ATTR_SELECTED_FREQUENCY_RESPONSE_ANALYSIS_AMPLITUDE_PROFILE_POINT);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolLINFrameRepCapBoolean_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolLINFrameRepCapBoolean_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolLINFrameFieldRepCapBoolean_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolLINFrameFieldRepCapBoolean_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolHBTOFrameFieldRepCapBoolean_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolHBTOFrameFieldRepCapBoolean_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolLINFrameFieldRepCapInt32_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolLINFrameFieldRepCapInt32_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolLINErrorRepCapBoolean_WriteCallback
Replaces <Error> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_ERROR
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolLINErrorRepCapBoolean_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        0, 0, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_ERROR);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolCANFrameFieldRepCapHexString_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANFrameFieldRepCapHexString_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolHBTOFrameFieldRepCapHexString_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolHBTOFrameFieldRepCapHexString_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPMIFrameFieldRepCapHexString_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPMIFrameFieldRepCapHexString_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolCANFrameRepCapBoolean_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANFrameRepCapBoolean_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolTNOSFrameRepCapBoolean_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolTNOSFrameRepCapBoolean_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolCANFrameFieldRepCapBoolean_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANFrameFieldRepCapBoolean_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolTNOSFrameFieldRepCapBoolean_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolTNOSFrameFieldRepCapBoolean_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FIELD, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolCANErrorRepCapBoolean_WriteCallback
Replaces <Error> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_ERROR
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANErrorRepCapBoolean_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        0, 0, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_ERROR);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolHBTOErrorRepCapBoolean_WriteCallback
Replaces <Error> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_ERROR
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolHBTOErrorRepCapBoolean_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        0, 0, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_ERROR);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPMIErrorRepCapBoolean_WriteCallback
Replaces <Error> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_ERROR
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPMIErrorRepCapBoolean_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalFrameFieldErrorRepCap_WriteCallback(io, repCapName, attr, value,
        0, 0, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_ERROR);
}


/****************************************************************************************************
 *  rsmxo_FormatFrameFieldHexStringRepCap_WriteCallback
Replaces <Frame> with value of ViAttr selectedFrame
Replaces <Field> with value of ViAttr selectedField
Prefixes the string with #H
    ******************************************************************************************************/
ViStatus rsmxo_FormatFrameFieldHexStringRepCap_WriteCallback(ViSession          instrSession,
                                                             ViConstString      repCapName,
                                                             RsCoreAttributePtr attr,
                                                             void*              value,
                                                             ViAttr             selectedFrame,
                                                             ViAttr             selectedField)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   aux[RS_MAX_SHORT_MESSAGE_BUF_SIZE];
    ViInt32  field, frame;
    ViInt32  binPattern = RSMXO_VAL_DATA_FORMAT_HEX;

    checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, RSMXO_ATTR_REMOTE_BITPATTERN_FORMAT, RS_VAL_DIRECT_USER_CALL, &binPattern));

    if (binPattern != RSMXO_VAL_DATA_FORMAT_HEX)
        checkErr(RsCore_SetAttributeViInt32(instrSession, NULL, RSMXO_ATTR_REMOTE_BITPATTERN_FORMAT, RS_VAL_DIRECT_USER_CALL, RSMXO_VAL_DATA_FORMAT_HEX));

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (selectedFrame > 0)
    {
        // Replaces <Frame> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedFrame, 0, &frame));
        snprintf(aux, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld", frame);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Frame>", aux, VI_TRUE);
    }

    if (selectedField > 0)
    {
        // Replaces <Field> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedField, 0, &field));
        snprintf(aux, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld", field);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Field>", aux, VI_TRUE);
    }

    if (attr->rangeTableDiscrete)
    {
        checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        switch (attr->dataType)
        {
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, " #H", (ViString)value, NULL);
            break;

        case RS_VAL_EVENT:
            break;
        case RS_VAL_UNKNOWN_TYPE:
        case RS_VAL_INT32:
        case RS_VAL_INT64:
        case RS_VAL_REAL64:
        case RS_VAL_BOOLEAN:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    if (binPattern != RSMXO_VAL_DATA_FORMAT_HEX)
        checkErr(RsCore_SetAttributeViInt32(instrSession, NULL, RSMXO_ATTR_REMOTE_BITPATTERN_FORMAT, RS_VAL_DIRECT_USER_CALL, binPattern));
    return error;
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolUARTFrameFieldRepCapHexString_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_FIELD
Prefaces the string with #H
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolUARTFrameFieldRepCapHexString_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return(rsmxo_FormatFrameFieldHexStringRepCap_WriteCallback(io, repCapName, attr, value, RSMXO_ATTR_PROTOCOL_UART_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_UART_SELECTED_FIELD));
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolTNOSFrameFieldRepCapHexString_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FIELD
Prefaces the string with #H
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolTNOSFrameFieldRepCapHexString_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return(rsmxo_FormatFrameFieldHexStringRepCap_WriteCallback(io, repCapName, attr, value, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FIELD));
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CFrameFieldRepCapHexString_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CFrameFieldRepCapHexString_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value)
{
    return(rsmxo_FormatFrameFieldHexStringRepCap_WriteCallback(instrSession, repCapName, attr, value, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FIELD));
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPIFrameFieldRepCapHexString_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FIELD
Prefaces the string with #H
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPIFrameFieldRepCapHexString_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return(rsmxo_FormatFrameFieldHexStringRepCap_WriteCallback(io, repCapName, attr, value, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FIELD));
}


/****************************************************************************************************
 *  rsmxo_FormatProtocolLINFrameFieldRepCapHexString_WriteCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FIELD
Prefaces the string with #H
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolLINFrameFieldRepCapHexString_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatFrameFieldHexStringRepCap_WriteCallback(io, repCapName, attr, value, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FIELD);
}


/****************************************************************************************************
 *  rsmxo_FormatUniversalAmplitudePointRepCap_ReadCallback
Replaces <AmplitudePoint> with value of ViAttr selectedFrame
******************************************************************************************************/
ViStatus rsmxo_FormatUniversalAmplitudePointRepCap_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            ViAttr             selectedAmplitudePoint)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   responseFixed[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  responseUnknownLen = NULL; // For string attribute, use dynamic buffer
    ViInt32  responseSize = 0;
    ViInt32  rangeTableValue = 0;
    ViChar*  p2response = NULL;
    ViInt32  amplitudePoint;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (selectedAmplitudePoint > 0)
    {
        // Replaces <AmplitudePoint> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedAmplitudePoint, 0, &amplitudePoint));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", amplitudePoint);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<AmplitudePoint>", aux, VI_TRUE);
    }

    if (attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING)
    {
        // Query the instrument, get the unlimited size response
        checkErr(
            RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
        // For string-type attributes remove trailing Whitespaces and pair quotations
        RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        p2response = responseUnknownLen;
    }
    else
    {
        // Query the instrument, get the limited size response - RS_MAX_MESSAGE_BUF_SIZE
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, responseFixed));
        p2response = responseFixed;
    }

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    if (attr->rangeTableDiscrete)
    {
        // Value has to converted to the range table value
        checkErr(
            RsCore_FindRangeTableDiscreteNumericValue(instrSession, attr->rangeTableDiscrete, p2response, &rangeTableValue));
        *(ViInt32*)attr->p2value = rangeTableValue;
    }
    else
    {
        // No range table, convert the response directly to the appropriate type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            *((ViInt32*)attr->p2value) = (ViInt32)RsCore_Convert_String_To_ViReal64(p2response);  // NOLINT(clang-diagnostic-bad-function-cast)
            break;
        case RS_VAL_INT64:
            *((ViInt64*)attr->p2value) = (ViInt64)RsCore_Convert_String_To_ViReal64(p2response); // NOLINT(clang-diagnostic-bad-function-cast)
            break;
        case RS_VAL_REAL64:
            *((ViReal64*)attr->p2value) = RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
            break;
        case RS_VAL_BOOLEAN:
            *((ViBoolean*)attr->p2value) = RsCore_Convert_String_To_Boolean(p2response);
            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

/****************************************************************************************************
 *  rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback
Replaces <Frame> with value of ViAttr selectedFrame
Replaces <Field> with value of ViAttr selectedField
Replaces <Word> with value of ViAttr selectedWord
Replaces <Byte> with value of ViAttr selectedByte
Replaces <Error> with value of ViAttr selectedError
******************************************************************************************************/
ViStatus rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            ViAttr             selectedFrame,
                            ViAttr             selectedField,
                            ViAttr             selectedWord,
                            ViAttr             selectedByte,
                            ViAttr             selectedError)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   responseFixed[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  responseUnknownLen = NULL; // For string attribute, use dynamic buffer
    ViInt32  responseSize = 0;
    ViInt32  rangeTableValue = 0;
    ViChar*  p2response = NULL;
    ViInt32  frame, field, _byte, _error;
    ViInt64  word;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (selectedFrame > 0)
    {
        // Replaces <Frame> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedFrame, 0, &frame));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", frame);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Frame>", aux, VI_TRUE);
    }

    if (selectedField > 0)
    {
        // Replaces <Field> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedField, 0, &field));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", field);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Field>", aux, VI_TRUE);
    }

    if (selectedWord > 0)
    {
        // Replaces <Word> with number
        checkErr(RsCore_GetAttributeViInt64(instrSession, NULL, selectedWord, 0, &word));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%lld", word);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Word>", aux, VI_TRUE);
    }

    if (selectedByte > 0)
    {
        // Replaces <Byte> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedByte, 0, &_byte));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", _byte);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Byte>", aux, VI_TRUE);
    }

    if (selectedError > 0)
    {
        // Replaces <Error> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedError, 0, &_error));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", _error);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Error>", aux, VI_TRUE);
    }

    if (attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING)
    {
        // Query the instrument, get the unlimited size response
        checkErr(
            RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
        // For string-type attributes remove trailing Whitespaces and pair quotations
        RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        p2response = responseUnknownLen;
    }
    else
    {
        // Query the instrument, get the limited size response - RS_MAX_MESSAGE_BUF_SIZE
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, responseFixed));
        p2response = responseFixed;
    }

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    if (attr->rangeTableDiscrete)
    {
        // Value has to converted to the range table value
        checkErr(
            RsCore_FindRangeTableDiscreteNumericValue(instrSession, attr->rangeTableDiscrete, p2response, &rangeTableValue));
        *(ViInt32*)attr->p2value = rangeTableValue;
    }
    else
    {
        // No range table, convert the response directly to the appropriate type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            *((ViInt32*)attr->p2value) = (ViInt32)RsCore_Convert_String_To_ViReal64(p2response);  // NOLINT(clang-diagnostic-bad-function-cast)
            break;
        case RS_VAL_INT64:
            *((ViInt64*)attr->p2value) = (ViInt64)RsCore_Convert_String_To_ViReal64(p2response); // NOLINT(clang-diagnostic-bad-function-cast)
            break;
        case RS_VAL_REAL64:
            *((ViReal64*)attr->p2value) = RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
            break;
        case RS_VAL_BOOLEAN:
            *((ViBoolean*)attr->p2value) = RsCore_Convert_String_To_Boolean(p2response);
            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CFrameRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CFrameRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolHBTOFrameRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolHBTOFrameRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolTNOSFrameRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolTNOSFrameRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CFrameFieldRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CFrameFieldRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FIELD, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolTNOSFrameFieldRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolTNOSFrameFieldRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FIELD, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolHBTOFrameFieldRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolHBTOFrameFieldRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FIELD, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPMIFrameFieldRepCapString_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPMIFrameFieldRepCapString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FIELD, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CFrameFieldRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CFrameFieldRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FIELD, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolHBTOFrameFieldRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolHBTOFrameFieldRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FIELD, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolTNOSFrameFieldRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolTNOSFrameFieldRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FIELD, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolCANFrameFieldRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANFrameFieldRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FIELD, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolCANFrameFieldRepCapReal64_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANFrameFieldRepCapReal64_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FIELD, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPMIFrameFieldRepCapReal64_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPMIFrameFieldRepCapReal64_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FIELD, 0, 0, 0);
}


/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CFrameRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CFrameRepCapReal64_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CFrameRepCapReal64_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolHBTOFrameRepCapReal64_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolHBTOFrameRepCapReal64_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CFrameRepCapString_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CFrameRepCapString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CFrameByteRepCapReal64_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME
Replaces <Byte> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_BYTE
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CFrameByteRepCapReal64_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME, 0, 0, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_BYTE, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CFrameByteRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME
Replaces <Byte> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_BYTE
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CFrameByteRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME, 0, 0, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_BYTE, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CFrameByteRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME
Replaces <Byte> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_BYTE
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CFrameByteRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME, 0, 0, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_BYTE, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolHBTOFrameByteRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME
Replaces <Byte> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_BYTE
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolHBTOFrameByteRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME, 0, 0, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_BYTE, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CErrorRepCapString_ReadCallback
Replaces <Error> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_ERROR
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CErrorRepCapString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, 0, 0, 0, 0, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_ERROR);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPIFrameRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPIFrameRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPIFrameFieldRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPIFrameFieldRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FIELD, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPIFrameFieldRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPIFrameFieldRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FIELD, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CErrorRepCapBoolean_ReadCallback
Replaces <Error> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_ERROR
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CErrorRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, 0, 0, 0, 0, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_ERROR);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPIErrorRepCapBoolean_ReadCallback
Replaces <Error> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_ERROR
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPIErrorRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, 0, 0, 0, 0, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_ERROR);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPIFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPIFrameRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPIFrameRepCapReal64_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPIFrameRepCapReal64_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPIFrameWordRepCapReal64_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME
Replaces <Word> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_WORD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPIFrameWordRepCapReal64_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME, 0, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_WORD, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPIFrameRepCapString_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPIFrameRepCapString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolUARTFrameRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolUARTFrameRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_UART_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolUARTFrameFieldRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolUARTFrameFieldRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_UART_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_UART_SELECTED_FIELD, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolUARTFrameFieldRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolUARTFrameFieldRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_UART_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_UART_SELECTED_FIELD, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolCANFrameRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANFrameRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPMIFrameRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPMIFrameRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolCANFrameFieldRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANFrameFieldRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FIELD, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolCANErrorRepCapBoolean_ReadCallback
Replaces <Error> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_ERROR
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANErrorRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, 0, 0, 0, 0, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_ERROR);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPMIErrorRepCapBoolean_ReadCallback
Replaces <Error> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_ERROR
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPMIErrorRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, 0, 0, 0, 0, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_ERROR);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolLINFrameRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolLINFrameRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolLINFrameFieldRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolLINFrameFieldRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FIELD, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPMIFrameFieldRepCapBoolean_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPMIFrameFieldRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FIELD, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolLINFrameFieldRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolLINFrameFieldRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FIELD, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPMIFrameFieldRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPMIFrameFieldRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FIELD, 0, 0, 0);
}


/****************************************************************************************************
 *  rsmxo_FormatProtocolLINErrorRepCapBoolean_ReadCallback
Replaces <Error> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_ERROR
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolLINErrorRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, 0, 0, 0, 0, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_ERROR);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolLINFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolLINFrameRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolTNOSFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolTNOSFrameRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPMIFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPMIFrameRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolLINFrameRepCapReal64_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolLINFrameRepCapReal64_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolTNOSFrameRepCapReal64_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolTNOSFrameRepCapReal64_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPMIFrameRepCapReal64_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPMIFrameRepCapReal64_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolLINFrameRepCapString_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolLINFrameRepCapString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPMIFrameRepCapString_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPMIFrameRepCapString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolLINFrameByteInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME
Replaces <Byte> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_BYTE
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolLINFrameByteRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME, 0, 0, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_BYTE, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolTNOSFrameByteInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME
Replaces <Byte> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_BYTE
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolTNOSFrameByteRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME, 0, 0, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_BYTE, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolLINFrameByteString_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME
Replaces <Byte> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_BYTE
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolLINFrameByteRepCapString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME, 0, 0, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_BYTE, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolHBTOFrameByteString_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME
Replaces <Byte> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_BYTE
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolHBTOFrameByteRepCapString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME, 0, 0, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_BYTE, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolTNOSFrameByteString_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME
Replaces <Byte> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_BYTE
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolTNOSFrameByteRepCapString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME, 0, 0, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_BYTE, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatFrameFieldHexStringRepCap_ReadCallback
Replaces <Frame> with value of ViAttr selectedFrame
Replaces <Field> with value of ViAttr selectedField
Removes #H from the string
******************************************************************************************************/
ViStatus rsmxo_FormatFrameFieldHexStringRepCap_ReadCallback(ViSession          instrSession,
                                                            ViConstString      repCapName,
                                                            RsCoreAttributePtr attr,
                                                            ViAttr             selectedFrame,
                                                            ViAttr             selectedField)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   payload[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];
    ViInt32  frame, field;
    ViInt32  binPattern = RSMXO_VAL_DATA_FORMAT_HEX;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, RSMXO_ATTR_REMOTE_BITPATTERN_FORMAT, RS_VAL_DIRECT_USER_CALL, &binPattern));
    if (binPattern != RSMXO_VAL_DATA_FORMAT_HEX)
        checkErr(RsCore_SetAttributeViInt32(instrSession, NULL, RSMXO_ATTR_REMOTE_BITPATTERN_FORMAT, RS_VAL_DIRECT_USER_CALL, RSMXO_VAL_DATA_FORMAT_HEX));

    // If no command is available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    if (!(attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING))
    {
        checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, "rsmxo_FormatFrameFieldHexStringRepCap_ReadCallback only accepts RS_VAL_STRING and RS_VAL_RAWSTRING data types"));
    }

    if (selectedFrame > 0)
    {
        // Replaces <Frame> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedFrame, 0, &frame));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", frame);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Frame>", aux, VI_TRUE);
    }

    if (selectedField > 0)
    {
        // Replaces <Field> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedField, 0, &field));
        snprintf(aux, RS_MAX_MESSAGE_BUF_SIZE, "%ld", field);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Field>", aux, VI_TRUE);
    }

    checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, response));
    checkErr(RsCore_Format_BinaryReadPattern(response, RS_MAX_MESSAGE_BUF_SIZE, payload));
    checkErr(RsCore_SetAttributeP2Value(instrSession, attr, payload));

Error:
    if (cmd)
        free(cmd);

    if (binPattern != RSMXO_VAL_DATA_FORMAT_HEX)
        checkErr(RsCore_SetAttributeViInt32(instrSession, NULL, RSMXO_ATTR_REMOTE_BITPATTERN_FORMAT, RS_VAL_DIRECT_USER_CALL, binPattern));

    return error;
}


/****************************************************************************************************
 *  rsmxo_FormatProtocolCANFrameFieldRepCapHexString_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANFrameFieldRepCapHexString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return (rsmxo_FormatFrameFieldHexStringRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FIELD));
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolHBTOFrameFieldRepCapHexString_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolHBTOFrameFieldRepCapHexString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return (rsmxo_FormatFrameFieldHexStringRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FIELD));
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolTNOSFrameFieldRepCapHexString_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolTNOSFrameFieldRepCapHexString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return (rsmxo_FormatFrameFieldHexStringRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FIELD));
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPMIFrameFieldRepCapHexString_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPMIFrameFieldRepCapHexString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return (rsmxo_FormatFrameFieldHexStringRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_SPMI_SELECTED_FIELD));
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolLINFrameFieldRepCapHexString_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolLINFrameFieldRepCapHexString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return (rsmxo_FormatFrameFieldHexStringRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_LIN_SELECTED_FIELD));
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolUARTFrameFieldRepCapHexString_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_FIELD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolUARTFrameFieldRepCapHexString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatFrameFieldHexStringRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_UART_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_UART_SELECTED_FIELD);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolUARTErrorRepCapBoolean_ReadCallback
Replaces <Error> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_ERROR
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolUARTErrorRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, 0, 0, 0, 0, RSMXO_ATTR_PROTOCOL_UART_SELECTED_ERROR);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolHBTOErrorRepCapBoolean_ReadCallback
Replaces <Error> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_ERROR
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolHBTOErrorRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, 0, 0, 0, 0, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_ERROR);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolTNOSErrorRepCapBoolean_ReadCallback
Replaces <Error> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_ERROR
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolTNOSErrorRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, 0, 0, 0, 0, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_ERROR);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolUARTWordRepCapBoolean_ReadCallback
Replaces <Word> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_WORD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolUARTWordRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, 0, 0, RSMXO_ATTR_PROTOCOL_UART_SELECTED_WORD, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolUARTWordRepCapReal64_ReadCallback
Replaces <Word> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_WORD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolUARTWordRepCapReal64_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, 0, 0, RSMXO_ATTR_PROTOCOL_UART_SELECTED_WORD, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolUARTWordRepCapInt32_ReadCallback
Replaces <Word> with value of RSMXO_ATTR_PROTOCOL_UART_SELECTED_WORD
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolUARTWordRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, 0, 0, RSMXO_ATTR_PROTOCOL_UART_SELECTED_WORD, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolCANFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANFrameRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolHBTOFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolHBTOFrameRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolCANFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANFrameRepCapReal64_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolCANFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
Replaces <Byte> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_BYTE
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANFrameByteRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME, 0, 0, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_BYTE, 0);
}


/****************************************************************************************************
 *  rsmxo_FormatProtocolCANFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolCANFrameRepCapString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolHBTOFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_CAN_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolHBTOFrameRepCapString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_HBTO_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolTNOSFrameRepCapInt32_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolTNOSFrameRepCapString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalFrameWordByteErrorRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_TNOS_SELECTED_FRAME, 0, 0, 0, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatFRANAmplitudePointRepCapReal64_ReadCallback
Replaces <AmplitudePoint> with value of RSMXO_ATTR_SELECTED_FREQUENCY_RESPONSE_ANALYSIS_AMPLITUDE_PROFILE_POINT
******************************************************************************************************/
ViStatus rsmxo_FormatFRANAmplitudePointRepCapReal64_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalAmplitudePointRepCap_ReadCallback(io, repCapName, attr,
        RSMXO_ATTR_SELECTED_FREQUENCY_RESPONSE_ANALYSIS_AMPLITUDE_PROFILE_POINT);
}


/****************************************************************************************************
 *  rsmxo_FormatProtocolI2CFrameFieldRepCapHexString_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FIELD
Strips the #H from the answer
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolI2CFrameFieldRepCapHexString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return (rsmxo_FormatFrameFieldHexStringRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_I2C_SELECTED_FIELD));
}

/****************************************************************************************************
 *  rsmxo_FormatProtocolSPIFrameFieldRepCapHexString_ReadCallback
Replaces <Frame> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME
Replaces <Field> with value of RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FIELD
Strips the #H from the answer
******************************************************************************************************/
ViStatus rsmxo_FormatProtocolSPIFrameFieldRepCapHexString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return (rsmxo_FormatFrameFieldHexStringRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FRAME, RSMXO_ATTR_PROTOCOL_SPI_SELECTED_FIELD));
}

/****************************************************************************************************
 *  rsmxo_FormatUniversalNodeContentRepCap_ReadCallback
Replaces <Node> with value of ViAttr selectedNode
Replaces <Content> with value of ViAttr selectedContent
******************************************************************************************************/
ViStatus rsmxo_FormatUniversalNodeContentRepCap_ReadCallback(ViSession          instrSession,
                                                             ViConstString      repCapName,
                                                             RsCoreAttributePtr attr,
                                                             ViAttr             selectedNode,
                                                             ViAttr             selectedContent)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   responseFixed[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   aux[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  responseUnknownLen = NULL; // For string attribute, use dynamic buffer
    ViInt32  responseSize = 0;
    ViInt32  rangeTableValue = 0;
    ViChar*  p2response = NULL;
    ViInt32  node, content;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (selectedNode > 0)
    {
        // Replaces <Node> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedNode, 0, &node));
        snprintf(aux, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld", node);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Node>", aux, VI_TRUE);
    }

    if (selectedContent > 0)
    {
        // Replaces <Content> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedContent, 0, &content));
        snprintf(aux, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld", content);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Content>", aux, VI_TRUE);
    }

    if (attr->dataType == RS_VAL_STRING || attr->dataType == RS_VAL_RAWSTRING)
    {
        // Query the instrument, get the unlimited size response
        checkErr(
            RsCore_QueryAttributeCommandUnknownLength(instrSession, attr, cmd, &responseUnknownLen, &responseSize));
        // For string-type attributes remove trailing Whitespaces and pair quotations
        RsCore_TrimString(responseUnknownLen, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);
        p2response = responseUnknownLen;
    }
    else
    {
        // Query the instrument, get the limited size response - RS_MAX_MESSAGE_BUF_SIZE
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, responseFixed));
        p2response = responseFixed;
    }

    // In simulation, end here. Do not touch the cache value.
    // It is read by the GetAttribute and returned to the caller.
    if (RsCore_Simulating(instrSession))
        goto Error;

    if (attr->rangeTableDiscrete)
    {
        // Value has to converted to the range table value
        checkErr(
            RsCore_FindRangeTableDiscreteNumericValue(instrSession, attr->rangeTableDiscrete, p2response, &rangeTableValue));
        *(ViInt32*)attr->p2value = rangeTableValue;
    }
    else
    {
        // No range table, convert the response directly to the appropriate type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            *((ViInt32*)attr->p2value) = (ViInt32)RsCore_Convert_String_To_ViReal64(p2response); // NOLINT(clang-diagnostic-bad-function-cast)
            break;
        case RS_VAL_INT64:
            *((ViInt64*)attr->p2value) = (ViInt64)RsCore_Convert_String_To_ViReal64(p2response); // NOLINT(clang-diagnostic-bad-function-cast)
            break;
        case RS_VAL_REAL64:
            *((ViReal64*)attr->p2value) = RsCore_Convert_String_To_ViReal64(p2response);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            checkErr(RsCore_SetAttributeP2Value(instrSession, attr, p2response));
            break;
        case RS_VAL_BOOLEAN:
            *((ViBoolean*)attr->p2value) = RsCore_Convert_String_To_Boolean(p2response);
            break;
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

Error:
    if (cmd)
        free(cmd);

    if (responseUnknownLen)
        free(responseUnknownLen);

    return error;
}

/****************************************************************************************************
*  rsmxo_FormatSmartGridNodeRepCapBoolean_ReadCallback
Replaces <Node> with value of RSMXO_ATTR_SMART_GRID_SELECTED_NODE
******************************************************************************************************/
ViStatus rsmxo_FormatSmartGridNodeRepCapBoolean_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalNodeContentRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_SMART_GRID_SELECTED_NODE, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatSmartGridNodeRepCapInt32_ReadCallback
Replaces <Node> with value of RSMXO_ATTR_SMART_GRID_SELECTED_NODE
******************************************************************************************************/
ViStatus rsmxo_FormatSmartGridNodeRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalNodeContentRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_SMART_GRID_SELECTED_NODE, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatSmartGridNodeRepCapReal64_ReadCallback
Replaces <Node> with value of RSMXO_ATTR_SMART_GRID_SELECTED_NODE
******************************************************************************************************/
ViStatus rsmxo_FormatSmartGridNodeRepCapReal64_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalNodeContentRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_SMART_GRID_SELECTED_NODE, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatSmartGridNodeContentRepCapString_ReadCallback
Replaces <Node> with value of RSMXO_ATTR_SMART_GRID_SELECTED_NODE
Replaces <Content> with value of RSMXO_ATTR_SMART_GRID_SELECTED_CONTENT
******************************************************************************************************/
ViStatus rsmxo_FormatSmartGridNodeContentRepCapString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalNodeContentRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_SMART_GRID_SELECTED_NODE, RSMXO_ATTR_SMART_GRID_SELECTED_CONTENTS);
}

/****************************************************************************************************
*  rsmxo_FormatSmartGridNodeContentRepCapInt32_ReadCallback
Replaces <Node> with value of RSMXO_ATTR_SMART_GRID_SELECTED_NODE
Replaces <Content> with value of RSMXO_ATTR_SMART_GRID_SELECTED_CONTENT
******************************************************************************************************/
ViStatus rsmxo_FormatSmartGridNodeContentRepCapInt32_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    return rsmxo_FormatUniversalNodeContentRepCap_ReadCallback(io, repCapName, attr, RSMXO_ATTR_SMART_GRID_SELECTED_NODE, RSMXO_ATTR_SMART_GRID_SELECTED_CONTENTS);
}

/****************************************************************************************************
 *  rsmxo_FormatUniversalNodeContentErrorRepCap_WriteCallback
Replaces <Node> with value of ViAttr selectedNode
Replaces <Content> with value of ViAttr selectedContent
    ******************************************************************************************************/
ViStatus rsmxo_FormatUniversalNodeContentErrorRepCap_WriteCallback(
                            ViSession          instrSession,
                            ViConstString      repCapName,
                            RsCoreAttributePtr attr,
                            void*              value,
                            ViAttr             selectedNode,
                            ViAttr             selectedContent)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;
    ViChar   aux[RS_MAX_SHORT_MESSAGE_BUF_SIZE];
    ViInt32  node, content;

    // If no command available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    // Command string building from repeated capabilities
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (selectedNode > 0)
    {
        // Replaces <Node> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedNode, 0, &node));
        snprintf(aux, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld", node);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Node>", aux, VI_TRUE);
    }

    if (selectedContent > 0)
    {
        // Replaces <Content> with number
        checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, selectedContent, 0, &content));
        snprintf(aux, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%ld", content);
        RsCore_ReplaceString(cmd, RS_MAX_MESSAGE_BUF_SIZE, "<Content>", aux, VI_TRUE);
    }

    if (attr->rangeTableDiscrete)
    {
        ViChar* p2cmdString = NULL;
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, attr->rangeTableDiscrete, *(ViInt32*)value, &p2cmdString));
        RsCore_StrCatDynamicString(instrSession, &cmd, " ", p2cmdString, NULL);
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            RsCore_StrCatDynamicViInt32(instrSession, &cmd, " ", *(ViInt32*)value, NULL);
            break;
        case RS_VAL_INT64:
            RsCore_StrCatDynamicViInt64(instrSession, &cmd, " ", *(ViInt64*)value, NULL);
            break;
        case RS_VAL_REAL64:
            RsCore_StrCatDynamicViReal64(instrSession, &cmd, " ", *(ViReal64*)value, NULL);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, " \'", (ViString)value, "\'");
            break;
        case RS_VAL_BOOLEAN:
            RsCore_StrCatDynamicViBoolean(instrSession, &cmd, " ", *(ViBoolean*)value, NULL);
            break;
        case RS_VAL_EVENT:
            break;
        case RS_VAL_UNKNOWN_TYPE:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}

/****************************************************************************************************
 *  rsmxo_FormatSmartGridNodeRepCapBoolean_WriteCallback
Replaces <Node> with value of RSMXO_ATTR_SMART_GRID_SELECTED_NODE
******************************************************************************************************/
ViStatus rsmxo_FormatSmartGridNodeRepCapBoolean_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalNodeContentErrorRepCap_WriteCallback(io, repCapName, attr, value, RSMXO_ATTR_SMART_GRID_SELECTED_NODE, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatSmartGridNodeRepCapInt32_WriteCallback
Replaces <Node> with value of RSMXO_ATTR_SMART_GRID_SELECTED_NODE
******************************************************************************************************/
ViStatus rsmxo_FormatSmartGridNodeRepCapInt32_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalNodeContentErrorRepCap_WriteCallback(io, repCapName, attr, value, RSMXO_ATTR_SMART_GRID_SELECTED_NODE, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatSmartGridNodeRepCapReal64_WriteCallback
Replaces <Node> with value of RSMXO_ATTR_SMART_GRID_SELECTED_NODE
******************************************************************************************************/
ViStatus rsmxo_FormatSmartGridNodeRepCapReal64_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalNodeContentErrorRepCap_WriteCallback(io, repCapName, attr, value, RSMXO_ATTR_SMART_GRID_SELECTED_NODE, 0);
}

/****************************************************************************************************
 *  rsmxo_FormatSmartGridNodeContentRepCapString_WriteCallback
Replaces <Node> with value of RSMXO_ATTR_SMART_GRID_SELECTED_NODE
Replaces <Content> with value of RSMXO_ATTR_SMART_GRID_SELECTED_CONTENTS
******************************************************************************************************/
ViStatus rsmxo_FormatSmartGridNodeContentRepCapString_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalNodeContentErrorRepCap_WriteCallback(io, repCapName, attr, value, RSMXO_ATTR_SMART_GRID_SELECTED_NODE, RSMXO_ATTR_SMART_GRID_SELECTED_CONTENTS);
}

/****************************************************************************************************
 *  rsmxo_FormatSmartGridNodeContentRepCapInt32_WriteCallback
Replaces <Node> with value of RSMXO_ATTR_SMART_GRID_SELECTED_NODE
Replaces <Content> with value of RSMXO_ATTR_SMART_GRID_SELECTED_CONTENTS
******************************************************************************************************/
ViStatus rsmxo_FormatSmartGridNodeContentRepCapInt32_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    return rsmxo_FormatUniversalNodeContentErrorRepCap_WriteCallback(io, repCapName, attr, value, RSMXO_ATTR_SMART_GRID_SELECTED_NODE, RSMXO_ATTR_SMART_GRID_SELECTED_CONTENTS);
}

/****************************************************************************************************
 *  rsmxo_FormatHexString_WriteCallback
Prefixes the string with '#H' without suffixing the length.
******************************************************************************************************/
ViStatus rsmxo_FormatHexString_WriteCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr, void* value)
{
    ViStatus error = VI_SUCCESS;
    ViInt32 binPattern = RSMXO_VAL_DATA_FORMAT_HEX;

    checkErr(RsCore_GetAttributeViInt32(io, NULL, RSMXO_ATTR_REMOTE_BITPATTERN_FORMAT, RS_VAL_DIRECT_USER_CALL, &binPattern));
    checkErr(RsCore_SetAttributeViInt32(io, NULL, RSMXO_ATTR_REMOTE_BITPATTERN_FORMAT, RS_VAL_DIRECT_USER_CALL, RSMXO_VAL_DATA_FORMAT_HEX));

    checkErr(RsCore_SpecialCallback_HexPatternWriteNoLength(io, repCapName, attr, value));

Error:
    if (binPattern != RSMXO_VAL_DATA_FORMAT_HEX)
        RsCore_SetAttributeViInt32(io, NULL, RSMXO_ATTR_REMOTE_BITPATTERN_FORMAT, RS_VAL_DIRECT_USER_CALL, binPattern);

    return error;
}

/****************************************************************************************************
 *  rsmxo_FormatHexString_ReadCallback
Strips the '#H' from the answer.
******************************************************************************************************/
ViStatus rsmxo_FormatHexString_ReadCallback(ViSession io, ViConstString repCapName, RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViInt32 binPattern = RSMXO_VAL_DATA_FORMAT_HEX;

    checkErr(RsCore_GetAttributeViInt32(io, NULL, RSMXO_ATTR_REMOTE_BITPATTERN_FORMAT, RS_VAL_DIRECT_USER_CALL, &binPattern));
    checkErr(RsCore_SetAttributeViInt32(io, NULL, RSMXO_ATTR_REMOTE_BITPATTERN_FORMAT, RS_VAL_DIRECT_USER_CALL, RSMXO_VAL_DATA_FORMAT_HEX));

    checkErr(RsCore_SpecialCallback_BinOrHexPatternRead(io, repCapName, attr));

Error:
    if (binPattern != RSMXO_VAL_DATA_FORMAT_HEX)
        RsCore_SetAttributeViInt32(io, NULL, RSMXO_ATTR_REMOTE_BITPATTERN_FORMAT, RS_VAL_DIRECT_USER_CALL, binPattern);

    return error;
}


/*****************************************************************************
 * Function: rsmxo_logging_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Returns value
 *           from cache
 *****************************************************************************/
ViStatus rsmxo_logging_ReadCallback(ViSession          instrSession,
                                    ViConstString      repCapName,
                                    RsCoreAttributePtr attr)
{
    return VI_SUCCESS;
}

/*****************************************************************************
 * Function: rsspecan_logging_WriteCallback
 * Purpose:  This function overrides standard calback funtion.
 *****************************************************************************/
ViStatus rsmxo_logging_WriteCallback(ViSession          instrSession,
                                     ViConstString      repCapName,
                                     RsCoreAttributePtr attr,
                                     void*              value)
{
    return VI_SUCCESS;
}

/*****************************************************************************
 * Function: rsmxo_empty_ReadCallback
 * Purpose:  This function overrides standard calback funtion. Returns value
 *           from cache
 *****************************************************************************/
ViStatus rsmxo_empty_ReadCallback(ViSession          instrSession,
                                  ViConstString      repCapName,
                                  RsCoreAttributePtr attr)
{
    return VI_SUCCESS;
}

/*****************************************************************************
 * Function: rsmxo_empty_WriteCallback
 * Purpose:  This function overrides standard calback funtion. Does not write
 *           anything to instrument, just updates the cache
 *****************************************************************************/
ViStatus rsmxo_empty_WriteCallback(ViSession          instrSession,
                                   ViConstString      repCapName,
                                   RsCoreAttributePtr attr,
                                   void*              value)
{
    return VI_SUCCESS;
}

/****************************************************************************************************
 *  rsmxo_quotedString_WriteCallback
For an attribute without a range table, it works as the standard RsCore_WriteCallback
For attributes with discrete range tables, it writes the command value in single apotrophes
******************************************************************************************************/
ViStatus rsmxo_quotedString_WriteCallback(ViSession          instrSession,
                                          ViConstString      repCapName,
                                          RsCoreAttributePtr attr,
                                          void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;

    // If no command is available, exit the function
    if (isNullOrEmpty(attr->command))
        goto Error;

    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    if (attr->rangeTableDiscrete)
    {
        ViChar* p2cmdString = NULL;
        checkErr(RsCore_FindRangeTableDiscreteCmdString(instrSession, attr->rangeTableDiscrete, *(ViInt32*)value, &p2cmdString));
        RsCore_StrCatDynamicString(instrSession, &cmd, " '", p2cmdString, "'");
    }
    else
    {
        // No range table, cmdString is composed based on the data type
        switch (attr->dataType)
        {
        case RS_VAL_INT32:
            RsCore_StrCatDynamicViInt32(instrSession, &cmd, " ", *(ViInt32*)value, NULL);
            break;
        case RS_VAL_INT64:
            RsCore_StrCatDynamicViInt64(instrSession, &cmd, " ", *(ViInt64*)value, NULL);
            break;
        case RS_VAL_REAL64:
            RsCore_StrCatDynamicViReal64(instrSession, &cmd, " ", *(ViReal64*)value, NULL);
            break;
        case RS_VAL_STRING:
        case RS_VAL_RAWSTRING:
            RsCore_StrCatDynamicString(instrSession, &cmd, " \'", (ViString)value, "\'");
            break;
        case RS_VAL_BOOLEAN:
            RsCore_StrCatDynamicViBoolean(instrSession, &cmd, " ", *(ViBoolean*)value, NULL);
            break;
        case RS_VAL_EVENT:
            break;
        case RS_VAL_UNKNOWN_TYPE:
        default:
            checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
        }
    }

    checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));

Error:
    if (cmd)
        free(cmd);

    return error;
}

/*****************************************************************************
 * Function: rsmxo_quotedInt_WriteCallback
 * Purpose:  This function overrides standard calback funtion. Writes string
 *           with apostrophes
 *****************************************************************************/
ViStatus rsmxo_quotedInt_WriteCallback(ViSession          instrSession,
                                       ViConstString      repCapName,
                                       RsCoreAttributePtr attr,
                                       void*              value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " '", value, "'");
}
